/**************************************************************************************************
 *
 * Ansteuerung der Hardware
 *
 **************************************************************************************************/

/******************************
 *
 * Wassertemperatur ermitteln
 *
 ******************************/
int GetTemp (void)
 {
  int wert;

	ADON = 1;	// enable A2D converter
	ADIE = 0;	// not interrupt driven
  CHS0 = 0;  // Kanal 0
  CHS1 = 0;
  CHS2 = 0;

  ADCS2 = 1;
  ADCS1 = 1;
  ADCS0 = 0;

  PCFG3 = 0; // Ref = Betriebsspannung
  PCFG2 = 0;
  PCFG1 = 0;
  PCFG0 = 0;

  ADFM = 1;  // Rechts justiert

	GODONE=1;
	while (GODONE);
	ADIF=0;

	wert = ADRESL + ADRESH * 256;

  ADCON0 = 0x00;
  ADCON1 = 0x07;
  SMP=1;
  CKE=0;

  /*
  inttostr (wert);
  put_string_usart (outbuf);
  put_string_usart (" ");
  */
  Delayms (1);

  return (wert);
 }

/**************************************************************************************************
 *
 * Abfrage des Pumpentasters
 *
 **************************************************************************************************/
char get_taster_pumpe (void)
 {
  return (!(PORTC & Taster_Pumpe));
 }

/**************************************************************************************************
 *
 * Abfrage des Bruehtasters
 *
 **************************************************************************************************/
char get_taster_kaffee (void)
 {
  return (!(PORTC & Taster_Kaffee));
 }

/**************************************************************************************************
 *
 * Abfrage des Durchflussensors
 *
 **************************************************************************************************/
char get_wasser (void)
 {
  return (PORTA & Schalter_Wassertakt);
 }

/**************************************************************************************************
 *
 * Abfrage des Nherungsschalters "Stempel unten"
 *
 **************************************************************************************************/
char get_stempel_unten (void)
 {
  return (PORTB & Schalter_StempelUnten);
 }

/**************************************************************************************************
 *
 * Abfrage des Hallsensors "Stempel fhrt"
 *
 **************************************************************************************************/
char get_stempel_faehrt (void)
 {
  return (!(PORTB & Schalter_Stempelfaehrt));
 }

/**************************************************************************************************
 *
 * Abfrage des Temperaturschalters
 *
 **************************************************************************************************/
char get_boiler (void)
 {
  return (PORTB & Schalter_Temperatur);
 }

/**************************************************************************************************
 *
 * Abfrage des Hall-Sensors "Deckel oben"
 *
 **************************************************************************************************/
char get_deckel_oben (void)
 {
  return (!(PORTB & Schalter_DeckelOben));
 }

/**************************************************************************************************
 *
 * Abfrage des Kontaktes "Schwenker in Brhposition"
 *
 **************************************************************************************************/
char get_schwenker_bruehen (void)
 {
  return (!(PORTB & Schalter_SchwenkerBruehen));
 }

/**************************************************************************************************
 *
 * Abfrage des Kontaktes "Schwenker in Fllposition"
 *
 **************************************************************************************************/
char get_schwenker_fuellen (void)
 {
  return (!(PORTB & Schalter_SchwenkerFuellen));
 }

/**************************************************************************************************
 *
 * Abfrage des Kontaktes "Schwenker in Ausgussposition"
 *
 **************************************************************************************************/
char get_schwenker_ausguss (void)
 {
  return (!(PORTE & Schalter_SchwenkerAusguss));
 }

/**************************************************************************************************
 *
 * Schaltet den Wasserkocher
 *
 **************************************************************************************************/
void boiler (char art)
 {
  if (art)
   {
    PORTE |= OutBoilerEin;
    Zustand = TRUE;
   } else
   {
    PORTE &= ~OutBoilerEin;
    Zustand = FALSE;
   }
 }

/**************************************************************************************************
 *
 * Schaltet die Pumpe
 *
 **************************************************************************************************/
void pumpe (char art)
 {
  if (art) PORTD |= OutPumpeEin; else PORTD &= ~OutPumpeEin;
 }

/**************************************************************************************************
 *
 * Fhrt den Stempel
 *
 **************************************************************************************************/
void stempel (char art, char dir)
 {
  // Richtung festlegen
  if (!dir) PORTD |= OutStempelRichtung; else PORTD &= ~OutStempelRichtung;

  // Kurze Pause fr das Relais, fals Motor eingeschaltet wurde
  if (art) Delayms (4);

  // Motor schalten
  if (art) PORTD |= OutStempelEin; else PORTD &= ~OutStempelEin;
 }

/**************************************************************************************************
 *
 * Verfhrt den Stempel
 *
 **************************************************************************************************/
void stempel_fahren (char art)
 {

  // Kaffee zusammenpressen
  if (art == bruehen)
   {
    stempel (TRUE, FALSE);				  // Stempel rausschieben
    while (Stempelschritt < StempelPresspos)
     { // Impulse zhlen
       while (get_stempel_faehrt () );
       while (!get_stempel_faehrt () );
       Stempelschritt++;
     }
    stempel (FALSE, FALSE);				  // Stempel abschalten

   }

  // Kaffee einfllen -> ganz einfahren
  if (art == fuellen)
   {
    stempel (TRUE, TRUE);						// Stempel reinholen
    while (!get_stempel_unten ());  // Warten, bis er unten angekommen ist
    stempel (FALSE, TRUE);          // Motor abschalten
    Stempelschritt = 0;							// Unterste Position kennzeichnen
   }

  // Kaffee rausdrcken -> ganz ausfahren
  if (art == ausgiessen)
   {
    stempel (TRUE, FALSE);				  // Stempel rausschieben
    while (Stempelschritt < StempelEndpos)
     { // Impulse zhlen
       while (get_stempel_faehrt () );
       while (!get_stempel_faehrt () );
       Stempelschritt++;
     }
    stempel (FALSE, FALSE);				  // Stempel abschalten
   }


 }

/**************************************************************************************************
 *
 * Fhrt den Deckel
 *
 **************************************************************************************************/
void deckel (char art)
 {
  if (art)
   { // Deckel schliessen
    PORTD &= ~OutDeckelRichtung;				// Richtung festlegen
    PORTD |= OutDeckelVoll;             // Volldampf voraus
    Delayms (100);											// Kurze Pause
    // PORTD |= OutDeckelHalb;             // Halbe Kraft voraus
    // PORTD &= ~OutDeckelVoll;            // Volldampf abschalten
   } else

   { // Deckel ffnen
    PORTD |= OutDeckelRichtung;
    PORTD |= OutDeckelVoll;            // Volldampf voraus
    PORTD &= ~OutDeckelHalb;           // Halbe Kraft abschalten
    while (! get_deckel_oben ());			 // Warten, bis offen
    PORTD &= ~OutDeckelVoll;           // Volldampf abschalten
   }

 }

/**************************************************************************************************
 *
 * Fhrt den Schwenker
 *
 **************************************************************************************************/
void schwenker (char position)
 {

  // Los, koch Kaffee !!
  if (position == bruehen)
   {
    PORTD &= ~OutSchwenkerRichtung;							// Richtung festlegen
    Delayms (10);																// Kurze Pause
    PORTD |= OutSchwenkerEin;                   // Motor ein
    while (! get_schwenker_bruehen () );        // Warte, bis Position erreicht ist
    Delayms (2);
    while (! get_schwenker_bruehen () );        // Warte, bis Position erreicht ist
    PORTD &= ~OutSchwenkerEin;                  // Motor aus
   }

  // Ausgiessen den Dreck...
  if (position == ausgiessen)
   {
    PORTD |= OutSchwenkerRichtung;							// Richtung festlegen
    Delayms (10);																// Kurze Pause
    PORTD |= OutSchwenkerEin;                   // Motor ein
    while (! get_schwenker_ausguss () );        // Warte, bis Position erreicht ist
    Delayms (2);
    while (! get_schwenker_ausguss () );        // Warte, bis Position erreicht ist
    PORTD &= ~OutSchwenkerEin;                  // Motor aus
   }

  // Befllen ist angesagt
  if (position == fuellen)
   {

    // Sind wir in Ausgussposition ?
    if (get_schwenker_ausguss ())
     {
       PORTD &= ~OutSchwenkerRichtung;						// Richtung festlegen
       Delayms (10);															// Kurze Pause
       PORTD |= OutSchwenkerEin;                  // Motor ein
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist
       Delayms (2);
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist

       while (get_schwenker_fuellen () );         // Warte, bis der Sensor wieder verlassen wird
       Delayms (2);
       while (get_schwenker_fuellen () );         // Warte, bis der Sensor wieder verlassen wird

       PORTD &= ~OutSchwenkerEin;                 // Motor aus
     } else

    // Oder in Brhposition ?
    if (get_schwenker_bruehen ())
     {
       PORTD |= OutSchwenkerRichtung;						// Richtung festlegen
       Delayms (10);															// Kurze Pause
       PORTD |= OutSchwenkerEin;                  // Motor ein
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist
       Delayms (2);
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist
       PORTD &= ~OutSchwenkerEin;                 // Motor aus

     } else

     // Offenbar weder noch
     {
       // Erstmal in Brhposition
       PORTD &= ~OutSchwenkerRichtung;							// Richtung festlegen
       Delayms (10);																// Kurze Pause
       PORTD |= OutSchwenkerEin;                   // Motor ein
       while (! get_schwenker_bruehen () );        // Warte, bis Position erreicht ist
       Delayms (2);
       while (! get_schwenker_bruehen () );        // Warte, bis Position erreicht ist
       PORTD &= ~OutSchwenkerEin;                  // Motor aus

       // Dann in Fllposition
       PORTD |= OutSchwenkerRichtung;						// Richtung festlegen
       Delayms (10);															// Kurze Pause
       PORTD |= OutSchwenkerEin;                  // Motor ein
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist
       Delayms (2);
       while (! get_schwenker_fuellen () );       // Warte, bis Position erreicht ist
       PORTD &= ~OutSchwenkerEin;                 // Motor aus

     }

   }

 }


