/**************************************************************************************************************
 *
 * Ms. Coffee, Brhgruppe
 *
 * (C) Fingers elektrische WElt
 *
 * Autor : Finger
 *
 * Controller : Microchip 18F452, 10 Mhz
 * Einstellungen:  OSC=HS, WDT=OFF, PWT=ON, CP=OFF, BOD=ON, LVP=OFF
 *
 * TAB-Space : 2
 * Editor : MI-Edit
 * Compiler : HI-TECH PIC-C 18
 *
 *************************************************************************************************************/

#include	<pic18.h>
#include <string.h>
#include	<ctype.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdio.h>

#include  "defines.h"                                     // globale Definitionen

/**************************************************************************************************
 *
 * Globale Variablen
 *
 **************************************************************************************************/
long Timerticks = 0;                      // Zeitmessung
char outbuf [30];                         // Rckgabepuffer fr INT -> ASCII-Wandlung
char wasserticker = 0;                    // Zeitschleife zur Gewichtsbestimmung
int wasserwert = 0;                       // Gemessener Timerwert
int Stempelschritt = 0;                   // Gemessene Position des Stempels

// Stempelpositionen
int StempelPresspos	= 15;                 // Pressposition des Stempels
int StempelEndpos	= 45;                   // Auswurfposition des Stempels

char Zustand = FALSE;                     // Zustand der Heizung

char usart_empfangspuffer [Empfangspuffergroesse];          // Puffer fr empfangene Zeichen
char usart_schreibzeiger = 0;                               // Zeiger auf den Empfangspuffer (schreiben)
char usart_lesezeiger = 0;                                  // Zeiger auf den Empfangspuffer (lesen)

char cdummy;                                                // Empfangsregister auslesen
char zeiger = 0;                                            // Lasst uns in die Zukunft schauen....


/*****************************************************************************************************************
 *
 * String nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void put_string_usart (const char *data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (FERR) cdummy = RCREG;
  if (OERR)
  	{
    	CREN=0;
    	CREN=1;
  	}
  	do
  	 {
  		while (!TRMT);
    	TXREG = *data;
  	 } while ( *data++ );
}

/*****************************************************************************************************************
 *
 * Char nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void put_char_usart (char data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (FERR) cdummy = RCREG;
  if (OERR)
  	{
    	CREN=0;
    	CREN=1;
  	}
   while (!TRMT);
   TXREG = data;
}

/*****************************************************************************************************************
 *
 * Zeichen aus USART abholen, liefert 0, falls kein Zeichen vorhanden
 *
 *****************************************************************************************************************/
char get_char_usart (void)
 {
  char dummy = 0;

  // Sind berhaupt Zeichen vorhanden ?
  if (usart_lesezeiger != usart_schreibzeiger)
   {
    dummy = usart_empfangspuffer [usart_lesezeiger];
    usart_lesezeiger++;
    if (usart_lesezeiger == Empfangspuffergroesse) usart_lesezeiger = 0;
    return (dummy);
   } else return (0);

 }

/*****************************************************************************************************************
 *
 * Puffer leeren
 *
 *****************************************************************************************************************/
void flush_usart (void)
 {
  usart_lesezeiger = usart_schreibzeiger;
 }


/*****************************************************************************************************************
 *
 * UART initialisieren
 *
 *****************************************************************************************************************/
void UART_Init (void)
 {
  // UART-Einstellungen
  SPBRG=16;       								                // USART: 38400 Baud bei 10MHZ - 8Bit-Mode mit BRGH=1
  TXSTA=0x24;									                    // mit BRGH=1
  RCSTA=0x90;
  RCIE=1;										                      // Receive-Interrupt ein
  RCIF=0;										                      // Flag=0;
 }

/***************************************************************************************************************
 *
 * Diverses
 *
 ***************************************************************************************************************/
#include "misc.c"

/*************************************************************************************************
 *
 * Zentrale, ausgelagerte ISR
 *
 *************************************************************************************************/
#include "isr.c"

/*************************************************************************************************
 *
 * Globale Initialisierung
 *
 *************************************************************************************************/
#include "init.c"

/************************************************************************
 *
 * HAL
 *
 ************************************************************************/
#include "hardware.c"

/************************************************************************
 *
 * Kurzdarstellung der Eingangsschalter
 *
 ************************************************************************/
void zeige_schalter (void)
 {
  if (get_stempel_unten ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_stempel_faehrt ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_boiler ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_deckel_oben ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_schwenker_bruehen ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_schwenker_fuellen ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');
  if (get_schwenker_ausguss ()) put_char_usart ('1'); else put_char_usart ('0');
  put_char_usart (' ');

  put_string_usart ("\n\r");
 }


/************************************************************************
 *
 * Zhlt Impulse vom Durchflusssensor und steuert die Wassermenge
 *
 ************************************************************************/
void zaehlewasserbecher (char zeit)
 {
  char i;

  for (i=0; i < zeit; i++)
   {
    Delayms (100);
   }

 }


/*************************************************************************
 *
 * Menanzeige
 *
 *************************************************************************/
void menue (void)
 {
  /*
  put_string_usart ("\n\rHandbedienung\n\n\r");
  put_string_usart ("1 Schalter zeigen\n\r");
  put_string_usart ("2 Deckel auf\n\r");

  Delayms (20);

  put_string_usart ("3 Deckel zu\n\r");
  put_string_usart ("4 Pumpe ein\n\r");
  put_string_usart ("5 Pumpe aus\n\r");

  Delayms (20);

  put_string_usart ("6 Boiler an\n\r");
  put_string_usart ("7 Boiler aus\n\r");
  put_string_usart ("8 Schwenker fuellen\n\r");

  Delayms (20);

  put_string_usart ("9 Schwenker ausgiessen\n\r");
  put_string_usart ("0 Schwenker bruehen\n\r");

  Delayms (20);

  put_string_usart ("a Stempel rein\n\r");
  put_string_usart ("b Stempel pressen\n\r");
  put_string_usart ("c Stempel auswerfen\n\r");

  Delayms (20);

  put_string_usart ("d Wasser zaehlen\n\r");
  put_string_usart ("e Becher fuellen\n\r");
  put_string_usart ("f Bruehen\n\r");
  */

 }

/********************
 *
 * Brht den Kaffee
 *
 ********************/
void bruehe_kaffee (void)
 {
   char zeichen = 0;
   char blowout = 0;

   deckel (FALSE);                       // Deckel ffnen
   stempel_fahren (fuellen);             // Stempel einfahren
   schwenker (bruehen);                  // Schwenker auf Brhen

//   put_string_usart ("Kacke");

   schwenker (fuellen);                  // Schwenker in Fllposition fahren

   // put_string_usart ("Scheisse");

   // Mhle starten, die liefert "1", wenn sie loslegt, und noch einmal "1", wenn sie fertig ist
   Delayms (3);

   put_string_usart ("d1700");

   // put_string_usart ("Mist");

//   Delayms (3);

   zeichen = get_char_usart ();
   while (zeichen != '1') zeichen = get_char_usart ();

   // Die Mhle schon mal ein wenig vormahlen lassen
   // zaehlewasserbecher (45);
   // zaehlewasserbecher (45);
   /*
   zaehlewasserbecher (45);
   zaehlewasserbecher (45);
   zaehlewasserbecher (45);
   zaehlewasserbecher (45);
   */

   // Derweil Wasser erhitzen
   boiler (TRUE);

   while ( (GetTemp () < Vorwaermung) && (get_char_usart () == 0) )                // Warten, bis das Wasser heiss ist
    {
     Delayms (10);
     /*
     inttostr (GetTemp ());
     put_string_usart (outbuf);
     put_string_usart (".");
     */
    }

   // put_string_usart (" Heiss ! ");

   boiler (FALSE);                       // Boiler abschalten

   // Kurz ein wenig Wasser pumpen, um den kalten Rest aus dem Brhbehlter rauszusplen
   pumpe (TRUE);                         // Pumpe ein
   zaehlewasserbecher (2);
   pumpe (FALSE);                        // Pumpe aus

   // Wasser kurz ausgiessen
   zaehlewasserbecher (2);               // Kurze Pinkelpause
   schwenker (ausgiessen);               // Schwenker auf entleeren
   zaehlewasserbecher (2);               // Kurze Pinkelpause
   schwenker (bruehen);                  // Schwenker auf Brhen

   boiler (TRUE);

   schwenker (fuellen);                  // Schwenker in Fllposition fahren

   // Kurz ein wenig Wasser pumpen, um den Brhbehlter wieder aufzufllen
   pumpe (TRUE);                         // Pumpe ein
   // zaehlewasserbecher (1);
   pumpe (FALSE);                        // Pumpe aus

   // Jetzt auf die Mhle warten. Derweil das Wasser temperieren.....
   zeichen = get_char_usart ();
   while (zeichen != '1')
    {
     zeichen = get_char_usart ();

     if (Zustand == FALSE)
     if (GetTemp () < (Achtzig-Hysterese))
      {
       boiler (TRUE);
       // put_string_usart (" AN ");
      }

     if (Zustand == TRUE)
     if (GetTemp () > (Achtzig+Hysterese))
      {
       boiler (FALSE);
       if (blowout == 0) blowout = 1;
       // put_string_usart (" AUS ");
      }

      if (blowout == 1)
       {
        // Wasser kurz ausgiessen
        zaehlewasserbecher (2);               // Kurze Pinkelpause
        // schwenker (ausgiessen);               // Schwenker auf entleeren
        // zaehlewasserbecher (2);               // Kurze Pinkelpause
        schwenker (bruehen);                  // Schwenker auf Brhen
        schwenker (fuellen);                  // Schwenker in Fllposition fahren
        blowout = 2;
       }

    }


   // Die Mhle ist durch mit ihrem Job, jetzt geht die Sache ab.........
   schwenker (bruehen);                  // Schwenker auf Brhen
   deckel (TRUE);                        // Deckel schliessen
   stempel_fahren (bruehen);             // Stempel auf Pressen fahren
   boiler (TRUE);                        // Weiter heizen

   // Noch mal ordentlich Dampf aufs Wasser !
   zaehlewasserbecher (10);

   pumpe (TRUE);                         // Pumpe ein und Kaffee brhen
   zaehlewasserbecher (10);              // Das reicht fr einen kleinen Becher....
   pumpe (FALSE);                        // Pumpe aus
   boiler (FALSE);                       // Heizung aus

   // put_string_usart ("Kaffee fertig\n\r");

   // Pause, bis der Wasserspiegel wieder abgesunken ist.....
   zaehlewasserbecher (2);

   deckel (FALSE);                       // Deckel ffnen

   // Pause, bis alle Tropfen abgefallen sind
   zaehlewasserbecher (3);

   deckel (FALSE);                       // Deckel ffnen
   schwenker (ausgiessen);               // Schwenker auf Ausguss fahren
   stempel_fahren (ausgiessen);          // Stempel auf Reinigen fahren

   // Reinigen

   stempel_fahren (fuellen);             // Stempel einfahren
   deckel (FALSE);                       // Deckel ffnen
   schwenker (bruehen);                  // Schwenker auf Brhen fahren

   // -> das wars Leute !

 }

/************************************************************************
 *
 * Handbedienung
 *
 ************************************************************************/
void hand (void)
 {
  char zeichen;

  menue ();

  while (1)
   {

    //----------------------------
    // Abfrage der ueren Taster
    //----------------------------

    // Soll Wasser gepumpt werden ?
    if (get_taster_pumpe ())
     {
      // Alles auf Splposition fahren
      deckel (FALSE);
      schwenker (ausgiessen);

      // put_string_usart ("Spuelposition\n\r");

      // Wasser pumpen, bis der Taster losgelassen wurde
      if (get_taster_pumpe ())
       {
        // put_string_usart ("Pumpe ein\n\r");
        pumpe (TRUE);
        Delayms (20);
       }

      while (get_taster_pumpe ());
      Delayms (20);
      pumpe (FALSE);

      // put_string_usart ("Pumpe aus\n\r");

     }

    // Soll ein Kaffee gebrht werden ?
    if (get_taster_kaffee ())
     {
      zeichen = get_char_usart ();
      while (zeichen != 0) zeichen = get_char_usart ();

      bruehe_kaffee ();

      zeichen = get_char_usart ();
      while (zeichen != 0) zeichen = get_char_usart ();

     }

    //-----------------------
    // Abfrage der Tastatur
    //-----------------------
    /*
    zeichen = get_char_usart ();
    if (zeichen != 0) put_char_usart (zeichen);

    switch (zeichen)
     {
      case '1' : {
                  zeige_schalter ();
                  put_string_usart ("Schalter zeigen\n\r");
                  break;
                 }

      case '2' : {
                  deckel (FALSE);
                  put_string_usart ("Deckel auf\n\r");
                  break;
                 }

      case '3' : {
                  deckel (TRUE);
                  put_string_usart ("Deckel zu\n\r");
                  break;
                 }

      case '4' : {
                  pumpe (TRUE);
                  put_string_usart ("Pumpe ein\n\r");
                  break;
                 }

      case '5' : {
                  pumpe (FALSE);
                  put_string_usart ("Pumpe aus\n\r");
                  break;
                 }

      case '6' : {
                  boiler (TRUE);
                  put_string_usart ("Boiler an\n\r");
                  break;
                 }

      case '7' : {
                  boiler (FALSE);
                  put_string_usart ("Boiler aus\n\r");
                  break;
                 }

      case '8' : {
                  schwenker (fuellen);
                  put_string_usart ("Schwenker fuellen\n\r");
                  break;
                 }

      case '9' : {
                  schwenker (ausgiessen);
                  put_string_usart ("Schwenker ausgiessen\n\r");
                  break;
                 }

      case '0' : {
                  schwenker (bruehen);
                  put_string_usart ("Schwenker bruehen\n\r");
                  break;
                 }

      case 'a' : {
                  stempel_fahren (fuellen);
                  put_string_usart ("Stempel rein\n\r");
                  break;
                 }

      case 'b' : {
                  // Eingabe der Position (3-stellig)

                  for (i=0; i<3; i++)
                   {
                    stelle [i] = get_char_usart ();
                    while (stelle [i] == 0) stelle [i] = get_char_usart ();
                   }
                  wert = ((stelle [0]-48) * 100) +
                         ((stelle [1]-48) * 10) +
                         ((stelle [2]-48));
                  // StempelPresspos = wert;
                  stempel_fahren (bruehen);
                  put_string_usart ("Stempel pressen\n\r");
                  break;
                 }

      case 'c' : {
                  stempel_fahren (ausgiessen);
                  put_string_usart ("Stempel auswerfen\n\r");
                  break;
                 }


      case 'd' : {
                  zaehlewasser ();
                  break;
                 }

      case 'f' : { // Einen Kaffee brhen
                  bruehe_kaffee ();
                  break;
                 }


     } // Von Switch

    zeichen = 0;
    */

   } // Von while (1)

 }


/************************************************************************
 *
 * Main
 *
 ************************************************************************/
void main (void)
{
  char zeichen;

  Init ();

  // Eventuellen Mll von der Mhle abfangen
  zeichen = get_char_usart ();
  while (zeichen != 0) zeichen = get_char_usart ();

  while (1) hand ();
}

// Thats all folks !

